import cv2
from flask import Flask, render_template, request
import threading
from dlib import get_frontal_face_detector, shape_predictor
import numpy as np
import face_swapping as fs
import time
import RPi.GPIO as GPIO

BUTTON_PIN = 20

# Imposta i pin di input GPIO
GPIO.setmode(GPIO.BCM)
GPIO.setup(BUTTON_PIN, GPIO.IN, pull_up_down=GPIO.PUD_UP)

app = Flask(__name__)

capture = cv2.VideoCapture(0)

isCapture = False
selected_effect = None
message_error = ""
thread_terminato = False
isKeep = False

def capture_image():

    global isCapture,capture,message_error,selected_effect,thread_terminato,seamlessclone,isKeep
    while True:
        ret, frame = capture.read()
        if ret:
            s = frame.copy()
            # Aggiungi la scritta al frame originale
            frame = cv2.resize(frame,(1000,1400))
            frame = cv2.flip(frame,1)
            cv2.putText(frame, message_error, (50, 50), cv2.FONT_HERSHEY_TRIPLEX , 0.8, (0, 0, 255),1)
            frame = cv2.rotate(frame, cv2.ROTATE_90_COUNTERCLOCKWISE)
            cv2.namedWindow("Video", cv2.WINDOW_NORMAL)
            cv2.setWindowProperty("Video", cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_FULLSCREEN)
            cv2.imshow("Video", frame)
            if isCapture:
                isCapture = False
                cv2.destroyAllWindows()
        # Controllo se il pulsante è stato premuto
        if cv2.waitKey(1) and GPIO.input(BUTTON_PIN) == GPIO.LOW:
            if ret:  # Verifica se il frame è valido
                cv2.destroyAllWindows()
                cv2.namedWindow("Foto", cv2.WINDOW_NORMAL)
                cv2.setWindowProperty("Foto", cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_FULLSCREEN)
                cv2.imwrite('foto.jpg', s)
                cv2.imshow("Foto", frame)
                isCapture = True
                isKeep = False
                while True:
                    if selected_effect is not None:
                        
                        image = cv2.imread('car.jpg')
                        image2 = cv2.imread('foto.jpg')
                        
                        resized1 = cv2.resize(image,(1080,1920))
                        resized2 = cv2.resize(image2,(1080,1920))
                        result = cv2.addWeighted(resized1,0.5,resized2, 0.5,0)
                        cv2.destroyAllWindows()
                        cv2.namedWindow("Waiting",cv2.WINDOW_NORMAL)
                        cv2.setWindowProperty("Waiting",cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_FULLSCREEN)
                        result = cv2.rotate(result,cv2.ROTATE_90_COUNTERCLOCKWISE)
                        cv2.imshow("Waiting",result)
                        cv2.waitKey(500)
                        apply_effect()
                        if message_error == "No Face detected":
                            seamlessclone = s
                                                
                        cv2.destroyAllWindows()
                        cv2.putText(seamlessclone, message_error, (50, 50), cv2.FONT_HERSHEY_TRIPLEX , 0.8, (0, 0, 255), 1)
                        seamlessclone = cv2.rotate(seamlessclone, cv2.ROTATE_90_COUNTERCLOCKWISE)
                        cv2.namedWindow("Result", cv2.WINDOW_NORMAL)
                        cv2.setWindowProperty("Result", cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_FULLSCREEN)
                        seamlessclone = cv2.resize(seamlessclone,(1920,1080))
                        cv2.imshow("Result", seamlessclone)
                        selected_effect = None
                    if cv2.waitKey(1) and isKeep or not isCapture:
                        message_error = ""
                        capture_image()
                        break

def apply_effect():
    global isCapture,message_error,selected_effect,thread_terminato,seamlessclone

    # Verifica se è stato selezionato un effetto
    img = cv2.imread("foto.jpg")
    img_gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)

    img2 = cv2.imread(f"static/{selected_effect}.jpeg")
    img2_gray = cv2.cvtColor(img2, cv2.COLOR_BGR2GRAY)

    detector = get_frontal_face_detector()
    predictor = shape_predictor("shape_predictor_68_face_landmarks.dat")
    
    height, width, channels = img2.shape
    img2_new_face = np.zeros((height, width, channels), np.uint8)

    # Face 1
    faces = detector(img_gray)
    if len(faces) == 0:
        message_error = "No Face detected"
        thread_terminato = True
        selected_effect = None
        return
    for face in faces:
        convexhull,landmarks_points,points = fs.get_landmarks_points(predictor,img_gray,face)
        indexes_triangles = fs.delaunay_triangulation(convexhull,landmarks_points,points)

    # Face 2
    faces2 = detector(img2_gray)
    for face in faces2:
        convexhull2,landmarks_points2,_ = fs.get_landmarks_points(predictor,img2_gray,face)

    fs.apply_triangular_warp(indexes_triangles, landmarks_points, landmarks_points2, img, img2_new_face)

    seamlessclone = fs.face_swap(img2,convexhull2,img2_new_face)

    thread_terminato = True
    selected_effect = None

@app.route('/')
def index():
    return render_template('index.html')

@app.route('/apply_effect', methods=['POST'])
def apply_effect_request():
    global selected_effect,isCapture,message_error

    if not isCapture:
        message_error = "Scatta la foto prima di appliacre l'effetto"
    else:
        data = request.get_json()
        effect = data.get('effect')
        selected_effect = effect
    return ""

def capture_button():
    global isKeep,isCapture
    while True:
        if GPIO.input(BUTTON_PIN) == GPIO.LOW:
           isKeep = True
        time.sleep(0.2)
           
cv2.destroyAllWindows()
# Creazione dei thread
c = threading.Thread(target=capture_button)
c.start()

d = threading.Thread(target=capture_image)
d.start()


# Avvio del server Flask
app.run(host='0.0.0.0', port=5000)

# Attendi la terminazione dei thread
capture_image_thread.join()

